/*
 * ExternalInterrupt.c
 *
 */
#include <avr/io.h>
#include <avr/interrupt.h>

//--------------------------------------------------------
// Heartbeat Timer
volatile uint8_t pulse_gen_timer = 0x00;

// interrupt routine: timer0 compare a
ISR(TIMER0_COMPA_vect) {
  if (pulse_gen_timer) {
    pulse_gen_timer--;
  }
}

//--------------------------------------------------------
// PulseCapture
enum eCaptureState {
  eWAIT_INT0_RISING_EDGE = 0,
  eWAIT_INT0_FALLING_EDGE
};

uint16_t channel0_count = 0x0000;
uint8_t i;

enum eCaptureState captureState = eWAIT_INT0_RISING_EDGE;
enum eCaptureState generateState = eWAIT_INT0_RISING_EDGE;

// INT0 change irq
ISR(INT0_vect) {
  // input set?
  if (PORTD & (1<<PORTD2)) {
    // rising edge
    if (captureState == eWAIT_INT0_RISING_EDGE) {
      // reset timer1
      TCNT1 = 0x0000;
      // next state
      captureState = eWAIT_INT0_FALLING_EDGE;
    }
    else {
      // wrong edge in this state
      channel0_count = 0x0000;
    }
  }
  else {
    // falling edge
    if (captureState == eWAIT_INT0_FALLING_EDGE) {
      // get timer1 value
      channel0_count = TCNT1;
      // next state
      captureState = eWAIT_INT0_RISING_EDGE;
    }
    else {
      // wrong edge in this state
      channel0_count = 0x0000;
    }
  }
}


int main(void)
{
  // init TIMER0
  OCR0A =  249;	  // 250 timer ticks until interrupt routine is called
  TCCR0A = 0x02;  // CTC mode, clear timer on compare match a
  TCCR0B = 0x03;  // timer0 prescaler: clk/64
  TIMSK0 = 0x02;  // enable timer0 output compare a irq

  // init TIMER1
  TCCR1A = 0x00;  // normal port operation
  TCCR1B = 0x01;  // normal port operation, no input capture, prescaler clk/1
  TCCR1C = 0x00;  // no force output compare

  // ext INT0
  EICRA = 0x01;   // INT0: any change -> interrupt
  EIMSK = 0x01;   // enable INT0 irq

	// init pulse capture outputs
	DDRD = (1<<PORTD2);
	PORTD = 0x00;

  // test output: port c
  DDRC = 0xff;

	sei();

	while(1) {
  	// generate capture pulses
  	if (!pulse_gen_timer) {
    	switch (generateState)
    	{
      case eWAIT_INT0_RISING_EDGE:
      	pulse_gen_timer = 2;
      	// set INT0 signal
      	PORTD |= (1<<PORTD2);
      	// next state
      	generateState = eWAIT_INT0_FALLING_EDGE;
      	break;
      case eWAIT_INT0_FALLING_EDGE:
      	pulse_gen_timer = 8;
      	// reset INT0 signal
      	PORTD &= ~(1<<PORTD2);
      	// next state
      	generateState = eWAIT_INT0_RISING_EDGE;
      	break;
      default:
      	generateState = eWAIT_INT0_RISING_EDGE;
      	break;
    	}
  	}

    // do something with captured values
    PORTC = (uint8_t)(channel0_count & 0xFF);
    PORTC = (uint8_t)((channel0_count>>8) & 0xFF);
	}
}